﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Windows;
using Ascon.Pilot.SDK.Menu;

namespace Ascon.Pilot.SDK.ContextMenuSample
{
    [Export(typeof(IMenu<ObjectsViewContext>))]
    public class ObjectListContextMenuPlugin : IMenu<ObjectsViewContext>
    {
        private const string MENU_ITEM_NAME = "PluginMenuItem";
        private const string DOCUMENT_MENU_ITEM_NAME = "DocumentPluginMenuItem";
        private const string CONTEXT_MENU_ITEM_NAME = "ContextPluginMenuItem";
        private const string MANY_SELECTED_MENU_ITEM_NAME = "ManySelectedMenuItemName";
        private const string SAVE_MENU_ITEM_NAME = "SaveExampleMenuItemName";

        private readonly IFileSaver _fileSaver;

        [ImportingConstructor]
        public ObjectListContextMenuPlugin(IFileSaver fileSaver)
        {
            _fileSaver = fileSaver;
        }

        public void Build(IMenuBuilder builder, ObjectsViewContext context)
        {
            var objects = context.SelectedObjects.ToList();
            var icon = IconLoader.GetIcon("menu_icon.svg");

            var itemNames = builder.ItemNames.ToList();
            const string indexItemName = "miShowSharingSettings";
            var insertIndex = itemNames.IndexOf(indexItemName);

            builder.AddSeparator(++insertIndex);
            // Show menu for the empty area within a folder
            if (context.IsContext)
            {
                builder.AddItem(CONTEXT_MENU_ITEM_NAME, ++insertIndex)
                       .WithHeader("Menu for empty area")
                       .WithIcon(icon);

                builder.AddItem("name", ++insertIndex)
                       .WithHeader("Disabled menu for empty area")
                       .WithIcon(icon)
                       .WithIsEnabled(false);

                builder.AddSeparator(++insertIndex);
                return;
            }

            // Show menu when a document is selected
            if (objects.Count == 1)
            {
                var selectedObject = objects.Single();
                if (selectedObject.Type.HasFiles)
                {
                    builder.AddItem(DOCUMENT_MENU_ITEM_NAME, ++insertIndex)
                           .WithHeader("Menu for document")
                           .WithIcon(icon);
                    builder.AddItem(SAVE_MENU_ITEM_NAME, insertIndex).WithHeader("Save file");
                }
                else
                {
                    builder.AddItem(MENU_ITEM_NAME, ++insertIndex)
                           .WithHeader("Menu for object")
                           .WithIcon(icon);
                }
            }

            // Show menu for many selected objects
            if (objects.Count > 1)
            {
                builder.AddItem(MANY_SELECTED_MENU_ITEM_NAME, ++insertIndex)
                       .WithHeader("Menu for many objects")
                       .WithIcon(icon);

                builder.AddItem("name", ++insertIndex)
                       .WithHeader("Disabled menu for many objects")
                       .WithIcon(icon)
                       .WithIsEnabled(false);
            }
            else
            {
                builder.AddItem("name", ++insertIndex)
                       .WithHeader("Disabled menu for object")
                       .WithIcon(icon)
                       .WithIsEnabled(false);
    
            }

            builder.AddSeparator(++insertIndex);
        }

        public void OnMenuItemClick(string name, ObjectsViewContext context)
        {
            if (MENU_ITEM_NAME == name)
                MessageBox.Show(string.Format("{0} was clicked", MENU_ITEM_NAME));

            if (DOCUMENT_MENU_ITEM_NAME == name)
                MessageBox.Show(string.Format("{0} was clicked", DOCUMENT_MENU_ITEM_NAME));

            if (CONTEXT_MENU_ITEM_NAME == name)
                MessageBox.Show(string.Format("{0} was clicked", CONTEXT_MENU_ITEM_NAME));

            if (MANY_SELECTED_MENU_ITEM_NAME == name)
                MessageBox.Show(string.Format("{0} was clicked", MANY_SELECTED_MENU_ITEM_NAME));

            if (name == SAVE_MENU_ITEM_NAME)
            {
                var selectedObject = context.SelectedObjects.First();
                if (selectedObject == null)
                    return;
                var outputFilePath = Path.GetTempFileName() + ".xps";
                var result = _fileSaver.SaveFile(selectedObject, outputFilePath);
                foreach (var res in result)
                {
                    MessageBox.Show(string.Format("Saved file {0}", res));
                }
            }
        }
    }
}
