﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Ascon.Pilot.SDK.TaskSample.TaskEditView
{
    class ModifyTaskEditViewModel : TaskEditViewModel, IObserver<IDataObject>
    {
        private readonly ITaskObject _task;
        private readonly string _oldTitle;
        private readonly string _oldDescription;
        private readonly List<Guid> _oldAttachments;
        private readonly int _oldExecutor;

        public ModifyTaskEditViewModel(ITaskObject task, ITaskModifier taskModifier, IObjectsRepository repository, IPilotDialogService pilotDialogService, ITabServiceProvider tabServiceProvider) 
            : base(taskModifier, repository, pilotDialogService, tabServiceProvider)
        {
            if (task == null) 
                throw new ArgumentNullException("task");

            _task = task;

            Title = _oldTitle = _task.DisplayTitle;
            Description = _oldDescription =_task.Description;

            _oldExecutor = _task.Executor != null ? _task.Executor.MainPosition.Position : -1;
            if (_oldExecutor != -1)
                ExecutorItems.Insert(0, new ExecutorItem(_task.Executor.MainPosition.Position, _task.Executor.DisplayName));

            foreach (var auditor in task.Auditors)
            {
                var person = _repository.GetPersonOnOrganisationUnit(auditor.Id);
                if (person != null)
                    AuditorsItems.Add(new ExecutorItem(person.Id, person.ActualName));
                else
                    AuditorsItems.Add(new ExecutorItem(auditor.Id, auditor.Title));
            }

            _oldAttachments = new List<Guid>(_task.InitiatorAttachments);
            LoadAttachments(_task.InitiatorAttachments);
        }

        public void LoadAttachments(IEnumerable<Guid> attachmentsIds)
        {
            _repository.SubscribeObjects(attachmentsIds).Subscribe(this);
        }

        public override bool SubmitTask()
        {
            if (!CanSubmit)
                return false;

            var auditorIds = _auditorsItems.Where(a => a.OrgUnitId != -1).Select(a => a.OrgUnitId).ToList();
            var builder = _taskModifier.EditTask(_task);
            builder.SetTitle(Title);
            builder.SetDescription(Description);
            builder.SetExecutor(ExecutorItems.First().OrgUnitId);
            builder.SetInitiatorAttachments(Attachments.Select(x => x.Id).ToList());
            builder.SetAuditors(auditorIds);
            _taskModifier.Apply();
            return true;
        }

        protected override void UpdateCanSubmit()
        {
            base.UpdateCanSubmit();
            
            if(!CanSubmit)
                return;

             var attachmens = _attachments.Select(x => x.Id).ToList();

            CanSubmit = _oldTitle != Title
                        || _oldDescription != Description
                        || _oldExecutor != ExecutorItems.First().OrgUnitId
                        || !_oldAttachments.SequenceEqual(attachmens);
        }

        protected override bool AllowMultipleExecutors
        {
            get { return false; }
        }

        public void OnNext(IDataObject value)
        {
            _attachments.Add(new TaskAttachment(value.Id, value.DisplayName, value.Type.Title, Theme.Icons.Instance.DocumentsIcon));
        }

        public void OnError(Exception error)
        {

        }

        public void OnCompleted()
        {

        }
    }
}
