﻿using System;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Windows;
using System.Xml.Serialization;
using Ascon.Pilot.SDK.GraphicLayerSample;
using Ascon.Pilot.SDK.Menu;
using Ascon.Pilot.SDK.XpsViewerSample.Properties;

namespace Ascon.Pilot.SDK.XpsViewerSample
{
    [Export(typeof(IMenu<XpsRenderClickPointContext>))]
    public class XpsRenderContextMenuSample : IMenu<XpsRenderClickPointContext>
    {
        private readonly IObjectModifier _modifier;
        private readonly IPerson _currentPerson;
        private const string CreateElement = "mCreateGraphicLayerElementAtPoint";

        [ImportingConstructor]
        public XpsRenderContextMenuSample(IObjectModifier modifier, IObjectsRepository repository)
        {
            _modifier = modifier;
            _currentPerson = repository.GetCurrentPerson();
        }

        public void Build(IMenuBuilder builder, XpsRenderClickPointContext context)
        {
            var items = builder.ItemNames.ToList();
            var position = items.Contains("miAddGraphicsLine") ? items.IndexOf("miAddGraphicsLine") : items.Count;
            builder.AddItem(CreateElement, position + 1)
                   .WithHeader(Resources.CreateGraphicLayerElementAtPoint)
                   .WithIsEnabled(context.SelectedVersion == context.DataObject.ActualFileSnapshot.Created);
        }

        public void OnMenuItemClick(string name, XpsRenderClickPointContext context)
        {
            if (name == CreateElement)
            {
                CreateGraphicLayerElementAtPoint(context);
            }
        }

        private void CreateGraphicLayerElementAtPoint(XpsRenderClickPointContext context)
        {
            var elementId = Guid.NewGuid();
            var positionId = _currentPerson.MainPosition.Position;
            var xamlContent = GraphicLayerElementCreator.CreateStampWithDateTime().ToString();
            var elementName = GraphicLayerElementConstants.GRAPHIC_LAYER_ELEMENT + elementId + "_" + positionId;
            var element = CreateGraphicLayerElement(context.ClickPoint, context.PageNumber, elementId, positionId);

            var builder = _modifier.Edit(context.DataObject);
            using (var graphicElementStream = new MemoryStream())
            {
                var serializer = new XmlSerializer(typeof(GraphicLayerElement));
                serializer.Serialize(graphicElementStream, element);
                graphicElementStream.Position = 0;
                builder.AddFile(elementName, graphicElementStream, DateTime.Now, DateTime.Now, DateTime.Now);
            }

            using (var contentStream = new MemoryStream())
            {
                using (var writer = new StreamWriter(contentStream))
                {
                    writer.Write(xamlContent);
                    writer.Flush();
                    builder.AddFile(GraphicLayerElementConstants.GRAPHIC_LAYER_ELEMENT_CONTENT + element.ContentId,
                        contentStream, DateTime.Now, DateTime.Now, DateTime.Now);
                }
            }
            _modifier.Apply();
        }

        private GraphicLayerElement CreateGraphicLayerElement(Point clickPoint, int pageNumber, Guid elementId, int positionId)
        {
            var scale = new Point(1, 1);
            return new GraphicLayerElement(elementId, Guid.NewGuid(), clickPoint.X, clickPoint.Y, positionId, scale,
                0, VerticalAlignment.Top, HorizontalAlignment.Left, GraphicLayerElementConstants.XAML, pageNumber, true);
        }
    }
}