﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;

namespace ProjectCloneWizard.ObjectCardView.States
{
    public static class StateConfigurationSerializer
    {
        public static string Serialize(UserStateConfiguration input)
        {
            var serializer = new XmlSerializer(typeof(UserStateConfiguration), GetKnownTypes());
            using (var stream = new MemoryStream())
            {
                serializer.Serialize(stream, input);
                return Encoding.UTF8.GetString(stream.ToArray());
            }
        }

        public static UserStateConfiguration Deserialize(string input)
        {
            if (string.IsNullOrEmpty(input))
                return new UserStateConfiguration();

            var serializer = new XmlSerializer(typeof(UserStateConfiguration), GetKnownTypes());
            var bytes = Encoding.UTF8.GetBytes(input);
            using (var stream = new MemoryStream(bytes))
            {
                try
                {
                    return (UserStateConfiguration)serializer.Deserialize(stream);
                }
                catch (Exception)
                {
                    return new UserStateConfiguration();
                }

            }
        }

        private static Type[] GetKnownTypes()
        {
            return new[] { typeof(UserStateConfiguration) };
        }
    }

    public class UserStateConfiguration
    {
        public List<Guid> UserStates { get; set; }

        public UserStateConfiguration()
        {
            UserStates = new List<Guid>();
        }
    }
}
