﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ascon.Pilot.SDK;
using ProjectCloneWizard.Common;
using ProjectCloneWizard.ObjectCardView;
using ProjectCloneWizard.Wizard;

namespace ProjectCloneWizard.ViewModels
{
    class ProjectCardPageViewModel : IWizardCaptionProvider, IWizardNavigationService
    {
        private readonly CreationInfo _creationInfo;
        private readonly ObjectCardViewModel _objectCardViewModel;

        public ProjectCardPageViewModel(CreationInfo creationInfo, IObjectsRepository repository, IPilotDialogService dialogService, IAttributeFormatParser attributeFormatParser)
        {
            _creationInfo = creationInfo;
            _objectCardViewModel = new ObjectCardViewModel(repository, dialogService, attributeFormatParser);
            _objectCardViewModel.IsValidInputChanged += OnIsValidInputChanged;
        }

        public ObjectCardViewModel ObjectCardViewModel => _objectCardViewModel;

        public void LoadCard()
        {
            _objectCardViewModel.CreateCard(_creationInfo.Template.Type);
        }

        public string GetTitle()
        {
            return "Шаг 4 из 4. Заполните карточку нового элемента";
        }

        public string GetTemplateName()
        {
            return string.Empty;
        }

        public bool CanGoNext()
        {
            return _objectCardViewModel.IsValidInput;
        }

        public bool CanGoBack()
        {
            return true;
        }

        public string NextButtonCaption()
        {
            return "Создать";
        }

        public event EventHandler UpdateState;

        public event EventHandler NavigateNext;

        private void OnIsValidInputChanged(object sender, EventArgs eventArgs)
        {
            UpdateState?.Invoke(this, EventArgs.Empty);
            _creationInfo.Attributes = _objectCardViewModel.Values;
        }

        public void Cancel()
        {
            
        }
    }
}
